------------------------------------------------------------------------
-- SuccessionDKP
------------------------------------------------------------------------
-- An extension of WebDKP allowing you to use the min/max/offspec DKP
-- model, while retaining the ability to sync with WebDKP.com
--
--
-- HOW THIS ADDON IS ORGANIZED:
-- The addon is grouped into a series of files which hold code for
-- specific functions. 
--
-- Core.lua       Core functions related to managing the addon
--
-- Whispers.lua   Handling various whisper functions (incl. bidding)
--
-- WebDKP.lua     Interaction with WebDKP
------------------------------------------------------------------------

SuccessionDKP = LibStub("AceAddon-3.0"):NewAddon("SuccessionDKP", "AceConsole-3.0", "AceEvent-3.0")

local options = {
   name = "SuccessionDKP",
   handler = SuccessionDKP,
   type = "group",
   args = {
      announceLoot = {
         type = "execute",
         name = "Announce Loot",
         desc = "Announces the loot of the boss",
         func = "announceLoot",
         order = 0,
      },
      announceBidders = {
         type = "execute",
         name = "Announce Bidders",
         desc = "Announces bidders for each item",
         func = "announceBidders",
         order = 1,
      },
      lootThreshold = {
         type = "select",
         name = "Loot Threshold",
         desc = "Minimum threshold for item loots",
         get = function(info) return tostring(SuccessionDKP.db.profile.lootThreshold) end,
         set = function(info, val) SuccessionDKP.db.profile.lootThreshold = val end,
         values = {
            ["0"] = "|cff9d9d9dPoor|r",
            ["1"] = "|cffffffffCommon|r",
            ["2"] = "|cff1eff00Uncommon|r",
            ["3"] = "|cff0070ddRare|r",
            ["4"] = "|cffa335eeEpic|r",
            ["5"] = "|cffff8000Legendary|r",
         },
         order = 2,
      },
      lootAnnounceChan = {
         type = "select",
         name = "Announce Channel",
         desc = "The channel in which to announce items looted",
         get = function(info) return tostring(SuccessionDKP.db.profile.lootAnnounceChan) end,
         set = function(info, val) SuccessionDKP.db.profile.lootAnnounceChan = val end,
         values = {
            ["SAY"] = "Say",
            ["EMOTE"] = "Emote",
            ["YELL"] = "Yell",
            ["PARTY"] = "Party",
            ["GUILD"] = "Guild",
            ["OFFICER"] = "Officer",
            ["RAID"] = "Raid",
            ["RAID_WARNING"] = "Raid Warning"
         },
         order = 3,
      },
   },
}

local defaultOptions = {
   profile = {
      lootThreshold = 3,
      lootAnnounceChan = "RAID",
      
      lootedItems = {}, -- An array of the items being distributed
      bidders = {},
   },
}
-- Called when the addon is loaded
function SuccessionDKP:OnInitialize()
   -- Setup our local database
   self.db = LibStub("AceDB-3.0"):New("SuccessionDKPDB", defaultOptions, "Default")
   
   -- Register our option table (see 'options' above)
   LibStub("AceConfig-3.0"):RegisterOptionsTable("SuccessionDKP", options)
   
   -- Add our options to the Blizzard's addon options menu
   self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SuccessionDKP", "SuccessionDKP")
   
   -- Register the chat commands to a ChatCommand function
   self:RegisterChatCommand("successiondkp", "ChatCommand")
   self:RegisterChatCommand("sdkp", "ChatCommand")
end

-- Called when the addon is enabled
function SuccessionDKP:OnEnable()
   -- Register to listen for the loot event
   SuccessionDKP:RegisterEvent("LOOT_OPENED")
   
   -- Listen for whispers
   SuccessionDKP:RegisterEvent("CHAT_MSG_WHISPER")
   
end

-- Called when the addon is disabled
function SuccessionDKP:OnDisable()
   
end

-- Called when the user inputs a slash command
function SuccessionDKP:ChatCommand(input)
   if (not input) or (input:trim() == "") then
      -- If we don't have input, open the addon GUI
      InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
   else
      -- Otherwise, handle the slash command
      LibStub("AceConfigCmd-3.0").HandleCommand(SuccessionDKP, "sdkp", "SuccessionDKP", input)
   end
end

function SuccessionDKP:LOOT_OPENED()
   self.db.profile.lootedItems = {}
   self.db.profile.bidders = {}
   
   self:Print("I just looted " .. UnitName("target") .. " and got " .. GetNumLootItems() .. " loot")
   
   local lootIndex = 1
   
   for index = 1, GetNumLootItems() do
      if (LootSlotIsItem(index)) then
         local _, _, quantity = GetLootSlotInfo(index)
         -- The loot is an item
         if (quantity == 1) then
            self:Print(index .. ")  " .. GetLootSlotLink(index))
            self.db.profile.lootedItems[lootIndex] = GetLootSlotLink(index)
         else
            self:Print(index .. ")  " .. GetLootSlotLink(index) .. "x" .. quantity)
            self.db.profile.lootedItems[lootIndex] = GetLootSlotLink(index) .. "x" .. quantity
         end
         self.db.profile.bidders[lootIndex] = {
            max = {},
            maxBids = 0,
            min = {},
            minBids = 0,
            off = {},
            offBids = 0,
         }
         lootIndex = lootIndex + 1
         self:Print("We now have " .. table.getn(self.db.profile.lootedItems) .. " items saved.")
      else
         -- The loot is coins
         local _, coins = GetLootSlotInfo(index)
         self:Print(index .. ")  " .. string.gsub(coins, "\n", " "))
      end
   end
end

function SuccessionDKP:announceLoot()
   for index = 1, table.getn(self.db.profile.lootedItems) do
      SendChatMessage(index .. ")  " .. self.db.profile.lootedItems[index], SuccessionDKP.db.profile.lootAnnounceChan)
   end
end

function SuccessionDKP:announceBidders()
   local dkpCost
   
   for itemIndex = 1, table.getn(self.db.profile.lootedItems) do
      SendChatMessage(itemIndex .. ")  " .. self.db.profile.lootedItems[itemIndex], SuccessionDKP.db.profile.lootAnnounceChan)
      
      -- Max bids
      for bidderIndex = 1, self.db.profile.bidders[itemIndex].maxBids do
         dkpCost = round(WebDKP_DkpTable[self.db.profile.bidders[itemIndex].max[bidderIndex]]["dkp"] * 0.25, 2)
         SendChatMessage("  (MAX) " .. self.db.profile.bidders[itemIndex].max[bidderIndex] .. " [" .. dkpCost .."]", SuccessionDKP.db.profile.lootAnnounceChan)
      end
      
      -- Min bids
      for bidderIndex = 1, self.db.profile.bidders[itemIndex].minBids do
         dkpCost = round (WebDKP_DkpTable[self.db.profile.bidders[itemIndex].max[bidderIndex]]["dkp"] * 0.10, 2)
         SendChatMessage("  (MIN) " .. self.db.profile.bidders[itemIndex].min[bidderIndex] .. " [" .. dkpCost .."]", SuccessionDKP.db.profile.lootAnnounceChan)
      end
      
      -- Off bids
      for bidderIndex = 1, self.db.profile.bidders[itemIndex].offBids do
         SendChatMessage("  (OFF) " .. self.db.profile.bidders[itemIndex].off[bidderIndex] .. " [5]", SuccessionDKP.db.profile.lootAnnounceChan)
      end
   end
end

function SuccessionDKP:CHAT_MSG_WHISPER(event, msg, author, _, status, _, _)
   
   local spaceSpot = string.find(msg, " ")
   local itemNumber = tonumber(string.sub(msg, 1, 1))
   
   -- Check to see if we have a valid item number
   if (spaceSpot == 2) then 
      if ((itemNumber >= 1) and (itemNumber <= table.getn(self.db.profile.lootedItems))) then                  
         -- Check to see if the bid amount is valid
         if (string.upper(string.sub(msg, 3, 5)) == "MAX") then
            -- max bid
            self:Print(author .. " bid MAX on " .. self.db.profile.lootedItems[itemNumber] .. ".")
            SendChatMessage("You have bid MAX on " .. self.db.profile.lootedItems[itemNumber], "WHISPER", nil, author)
            SendChatMessage("To retract your bid, reply with 'retract'.", "WHISPER", nil, author)
            self.db.profile.bidders[itemNumber].maxBids = self.db.profile.bidders[itemNumber].maxBids + 1
            self.db.profile.bidders[itemNumber].max[self.db.profile.bidders[itemNumber].maxBids] = author
            return
         end
         if (string.upper(string.sub(msg, 3, 5)) == "MIN") then
            -- min bid
            self:Print(author .. " bid MIN on " .. self.db.profile.lootedItems[itemNumber] .. ".")
            SendChatMessage("You have bid MIN on " .. self.db.profile.lootedItems[itemNumber], "WHISPER", nil, author)
            SendChatMessage("To retract your bid, reply with 'retract'.", "WHISPER", nil, author)
            self.db.profile.bidders[itemNumber].minBids = self.db.profile.bidders[itemNumber].minBids + 1
            self.db.profile.bidders[itemNumber].min[self.db.profile.bidders[itemNumber].minBids] = author
            return
         end
         if (string.upper(string.sub(msg, 3, 5)) == "OFF") then
            -- offspec bid
            self:Print(author .. " bid OFF on " .. self.db.profile.lootedItems[itemNumber] .. ".")
            SendChatMessage("You have bid OFF on " .. self.db.profile.lootedItems[itemNumber], "WHISPER", nil, author)
            SendChatMessage("To retract your bid, reply with 'retract'.", "WHISPER", nil, author)
            self.db.profile.bidders[itemNumber].offBids = self.db.profile.bidders[itemNumber].offBids + 1
            self.db.profile.bidders[itemNumber].off[self.db.profile.bidders[itemNumber].offBids] = author
            return
         end
         
         -- Invalid bid amount
         SendChatMessage("Invalid bid amount: " .. string.sub(msg, 3, 5) .. ". Valid amounts: MAX, MIN, OFF.", "WHISPER", nil, author)
      else
         -- Invalid item number
         SendChatMessage("Invalid item: " .. itemNumber, "WHISPER", nil, author);
      end
   end
end

function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end